package com.abacus.improved;

import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.TreeMap;

import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;

import com.abacus.legacy.BigIntegerBean;
import com.abacus.legacy.FileData;
import com.abacus.legacy.Node;
/**
 * This purpose of this class is to run the main interface in a frame.
 * @author saiumesh
 *
 */
public class AbacusMachineMainUI extends JFrame implements ActionListener{
	private final JMenu fileMenu = new JMenu("File");
	private final JMenu registerMenu = new JMenu("Modify");
	private final JMenuItem loadMachine = new JMenuItem("Load Machine");
	private final JMenuItem saveMachine = new JMenuItem("Save Machine");
	private final JMenuItem modify = new JMenuItem("Modify Initial Register Contents");
	private final JMenuItem exit = new JMenuItem("Exit");
	private FileData machineFile;
	private MachinePanel machinePanel;
	private RegistryEditorDialog regEdit;
	
	public AbacusMachineMainUI() {
		this.setTitle("iAMG - Improved Abacus Machine Generator");
		JMenuBar menu = new JMenuBar();
		this.machineFile = new FileData();
		this.regEdit = new RegistryEditorDialog();
		this.machinePanel = new MachinePanel(this);
		
		this.machinePanel.setPreferredSize(new Dimension(700, 600));
		this.machinePanel.setMinimumSize(new Dimension(700, 600));
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		this.setLocation(Toolkit.getDefaultToolkit().getScreenSize().width/2 - 300, 50);
		this.add(machinePanel);
		fileMenu.add(this.loadMachine);
		fileMenu.add(this.saveMachine);
		fileMenu.addSeparator();
		fileMenu.add(this.exit);
		registerMenu.add(modify);
		menu.add(fileMenu);
		menu.add(registerMenu);
		this.setJMenuBar(menu);
		loadMachine.addActionListener(this);
		saveMachine.addActionListener(this);
		exit.addActionListener(this);
		modify.addActionListener(this);
		
		this.pack();
		
		//TODO: add windowlistener to confirm closing of program
	}
	/**
	 * Used by SimulatorPanel in order to determine how many Player threads to create.
	 * @return size of panel list.
	 */
	public int getInputCount() {
		return regEdit.getPanelCount();
	}
	
	public RegistryEditorDialog getDialog() {
		return this.regEdit;
	}
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		AbacusMachineMainUI instance = new AbacusMachineMainUI();
		instance.setVisible(true);
		try {
			
		} catch(Exception ex) {
			System.out.println(ex.getMessage());
			System.out.println(ex.toString());
			System.exit(0);
		}
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if(e.getSource() == loadMachine) {
			loadMachine();
		} else if (e.getSource() == saveMachine) {
			if(machinePanel.getNodeList().size() > 0) {
				machineFile.nodes = machinePanel.getNodeList();
				machineFile.regs = (TreeMap) regEdit.regMap.values().toArray()[0];
				this.machineFile.save();
			} else
				JOptionPane.showMessageDialog(this,
					    "Machine does not contain any nodes",
					    "Cannot save machine..",
					    JOptionPane.ERROR_MESSAGE);
		} else if (e.getSource() == exit) {
			System.exit(0);
		} else if (e.getSource() == modify) {
			this.regEdit.setVisible(true);
		}
	}
	
	/**
	 * Load existing machine from file.
	 */
	private void loadMachine() {
		if(!machinePanel.isMachineRunning() &&machineFile.load()) {
			machinePanel.setNodeList((ArrayList<Node>) machineFile.nodes);
			machinePanel.setRegisterList((TreeMap<Integer, BigIntegerBean>) machineFile.regs);
			regEdit.repaint();
			machinePanel.repaint();
		}
	}

}
