package com.abacus.improved;

import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.Image;
import java.awt.image.ImageObserver;
import java.awt.image.MemoryImageSource;
import java.awt.image.PixelGrabber;

import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JToggleButton;

/**
 * The purpose of this class is to create a panel for the user to
 * manipulate the machine's nodes.
 * 
 * Many of the functions in this class exist to manipulate the resource images
 * @author saiumesh
 *
 */
public class ButtonPanel extends JPanel {
	//buttons
	private JToggleButton addState;
	private JToggleButton subState;
	private JToggleButton modifyState;
	private JToggleButton deleteState;
	
	//state integers
	public final static Integer ADDSTATE = 0;
	public final static Integer SUBSTATE = 1;
	public final static Integer MODSTATE = 2;
	public final static Integer DELSTATE = 3;

	//button images
	public final static Image imageAdd = makeRedTransparent((createImageIcon("com/abacus/images/imageAdd.GIF")).getImage());
	public final static Image imageSub = makeRedTransparent((createImageIcon("com/abacus/images/imageSub.GIF")).getImage());
	private final Image imageMod = makeRedTransparent((createImageIcon("com/abacus/images/select.gif")).getImage());
	private final Image imageDel = makeRedTransparent((createImageIcon("com/abacus/images/imageDel.gif")).getImage());
	
	public final static Image transparentAddImage = makeHazy(imageAdd);
	public final static Image transparentSubImage = makeHazy(imageSub);
	
	public ButtonPanel() {
		this.addState = new JToggleButton(new ImageIcon(imageAdd));
		this.subState = new JToggleButton(new ImageIcon(imageSub));
		this.modifyState = new JToggleButton(new ImageIcon(imageMod));
		this.deleteState = new JToggleButton(new ImageIcon(imageDel));
		ButtonGroup group = new ButtonGroup(); //this object takes care of switching between adding/modding nodes
		group.add(addState);
		group.add(subState);
		group.add(modifyState);
		group.add(deleteState);
		
		FlowLayout f = new FlowLayout();
		f.setAlignment(FlowLayout.CENTER);
		this.setLayout(f);
		this.setBackground(new Color(0, 127, 255));
		
		addState.setSelected(true);
		addState.setToolTipText("Insert Add State");
		this.add(addState);
		subState.setToolTipText("Insert Subtract State");
		this.add(subState);
		modifyState.setToolTipText("Modify States");
		this.add(modifyState);
		deleteState.setToolTipText("Delete Node");
		this.add(deleteState);
	}
	
	private static ImageIcon createImageIcon(String path) {
		java.net.URL imgURL = ButtonPanel.class.getClassLoader().getResource(path);
	    if (imgURL != null) {
	        return new ImageIcon(imgURL);
	    } else {
	        System.err.println("Couldn't find file: " + path);
	        return null;
	    }
	}
	
	public int getState() {
		if(addState.isSelected())
			return ADDSTATE;
		else if(subState.isSelected())
			return SUBSTATE;
		else if(modifyState.isSelected())
			return MODSTATE;
		else
			return DELSTATE;
	}
	
	/**
	 * All of the resource images have a red background (255, 0, 0). This method 
	 * removes the background and displays the non-red parts on the button.
	 * @param source
	 * @return
	 */
	public static Image makeRedTransparent(Image source) {
		return makeTransparent(source, Color.red);
	}
	public static Image makeTransparent(Image source, Color c)
	{
		int w = source.getWidth(null);
		int h = source.getHeight(null);

		int[] cols = getAllColors(source);

		for (int x = 0; x < w; ++x)
			for (int y = 0; y < h; ++y)
			{
				int color = cols[x + y * w];

				int[] rgb = getRGB(color);
				
				if (Math.abs(rgb[0] - c.getRed()) < 5 && 
					Math.abs(rgb[1] - c.getGreen()) < 5 && 
					Math.abs(rgb[2] - c.getBlue()) < 5)
				{ // red					
						
					cols[x + y * w] = 0;
				}
				else
					cols[x + y * w] = (255 << 24) | (rgb[0] << 16)
							| (rgb[1] << 8) | rgb[2];
			}

		return (new JPanel()).createImage(new MemoryImageSource(w, h, cols, 0,
				w));

	}
	
	public static int[] getAllColors(Image theImage)
	{
		int w = theImage.getWidth(null);
		int h = theImage.getHeight(null);

		int[] storeHere = new int[w * h];
		PixelGrabber pg = new PixelGrabber(theImage, 0, 0, w, h, storeHere, 0,
				w);
		try
		{
			pg.grabPixels();
		}
		catch (InterruptedException e)
		{
			JOptionPane.showMessageDialog(null,
					"interrupted waiting for pixels!");
			storeHere = null;
			return null;
		}
		if ((pg.getStatus() & ImageObserver.ABORT) != 0)
		{
			JOptionPane.showMessageDialog(null,
					"image fetch aborted or errored");
			storeHere = null;
			return null;
		}

		return storeHere;
	}

	public static int[] getRGB(int pixel) {
		int[] rgb = new int[3];

		rgb[0] = (pixel >> 16) & 0xff;
		rgb[1] = (pixel >> 8) & 0xff;
		rgb[2] = (pixel) & 0xff;

		return rgb;
	}
	
	public static Image makeHazy(Image source) {
		int w = source.getWidth(null);
		int h = source.getHeight(null);

		// System.out.println("width = " + w);

		int[] cols = getAllColors(source);

		for (int x = 0; x < w; ++x)
			for (int y = 0; y < h; ++y)
			{
				int color = cols[x + y * w];

				int[] rgb = getRGB(color);
				if ((color & 0xFF000000) != 0) // make this guy hazy
				{
					cols[x + y * w] = (96 << 24) | (rgb[0] << 16)
					| (rgb[1] << 8) | rgb[2];
				}
					
			}

		return (new JPanel()).createImage(new MemoryImageSource(w, h, cols, 0,
				w));

	}

}
