package com.abacus.improved;

import java.awt.BorderLayout;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.TreeMap;

import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import com.abacus.legacy.BigIntegerBean;
import com.abacus.legacy.Node;
/**
 * Holds the node, simulator, and editor panel. This class acts as a middle-man to send information between these panels and the 
 * rest of the program.
 * @author saiumesh
 *
 */
public class MachinePanel extends JPanel implements ChangeListener{
	NodeEditorPanel editor;
	private ButtonPanel btnPnl;
	private ZoomPanel zoomPnl;
	ArrayList<Node> nodeList;
	private TreeMap<Integer, BigIntegerBean> registerList;
	private JTabbedPane tabPnl;
	AbacusMachineMainUI parent;
	private SimulatorPanel simPnl;
	
	public MachinePanel(AbacusMachineMainUI parentFrame) {
		this.nodeList = new ArrayList<Node>();
		this.editor = new NodeEditorPanel(this);
		this.parent = parentFrame;
		this.simPnl = new SimulatorPanel(parent.getDialog());
		this.btnPnl = new ButtonPanel();
		this.zoomPnl = new ZoomPanel();
		this.registerList = new TreeMap<Integer, BigIntegerBean>();
		this.tabPnl = new JTabbedPane();
		this.tabPnl.addTab("Nodes", btnPnl);
		this.tabPnl.addTab("Zoom", zoomPnl);
		this.tabPnl.addTab("Simulator", simPnl);
		this.tabPnl.setTabLayoutPolicy(JTabbedPane.SCROLL_TAB_LAYOUT);
		
		tabPnl.addChangeListener(this);
		
		this.setLayout(new BorderLayout());
		this.add(tabPnl, BorderLayout.NORTH);
		this.add(editor, BorderLayout.CENTER);
	}
	
	public void setNodeList(ArrayList<Node> newNodeList) {
		this.nodeList = newNodeList;
		editor.nodes = nodeList;
		//if the editor pnl is locked, then this is not needed...just to be safe though
		if (!simPnl.isRunning())
			simPnl.setNodeList(nodeList);
	}
	public ArrayList<Node> getNodeList() {
		// TODO Auto-generated method stub
		return nodeList;
	}
	public TreeMap<Integer, BigIntegerBean> getRegisterList() {
		return this.registerList;
	}
	public void setRegisterList(TreeMap<Integer, BigIntegerBean> regs) {
		this.registerList = regs;
	}
	
	public boolean isNodeTabSelected() {
		return this.tabPnl.getSelectedComponent() instanceof ButtonPanel;
	}
	
	public boolean isMachineRunning() {
		return (simPnl != null && simPnl.isRunning());
	}
	
	/**
	 * Returns tab panel state to NodeEditorPanel in order to manipulate it (modify node state vs zooming)
	 * appropriately.
	 * @return
	 */
	public int getState() {
		if (this.tabPnl != null) {
			if(this.tabPnl.getSelectedComponent() instanceof ButtonPanel)
				return this.btnPnl.getState();
			else if (this.tabPnl.getSelectedComponent() instanceof ZoomPanel)
				return (-1 * this.zoomPnl.getZoomState());
		}
		return -1;
	}
	
	public void lockPanel() {
		editor.lockPanel();
	}
	
	public void unlockPanel() {
		editor.unlockPanel();
		editor.clearSelection();
	}

	@Override
	public void stateChanged(ChangeEvent arg0) {
		if(tabPnl.getSelectedComponent() == simPnl && !simPnl.isRunning()) {
			if(nodeList.size() > 0) {
				simPnl.setParent(this);
				parent.getDialog().lock(); //lock dialog to prevent further changes when running the simulator.
				simPnl.setDialog(parent.getDialog());
			} else {
				JOptionPane.showMessageDialog(new JFrame(), "Please create a machine before running a sim",
						"Error", JOptionPane.ERROR_MESSAGE);
				tabPnl.setSelectedComponent(btnPnl);
			}
        } else if(tabPnl.getSelectedComponent() != simPnl && !simPnl.isRunning()){
        	parent.getDialog().unlock();
        }
	}

}
