package com.abacus.improved;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Point;
import java.awt.Polygon;
import java.awt.RenderingHints;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.geom.AffineTransform;
import java.awt.geom.RoundRectangle2D;
import java.math.BigInteger;
import java.util.Date;
import java.util.HashMap;
import java.util.Set;
import java.util.TreeMap;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.SpringLayout;

import com.abacus.legacy.BigIntegerBean;
import com.abacus.resources.SpringUtilities;

public class RegistryEditorDialog extends JFrame implements MouseMotionListener, MouseListener, 
ActionListener{
	private JPanel inputPanel;
	RegisterPanel regPanel = new RegisterPanel(0);
	private static final int scrollButtonWidth = 40;
	public HashMap<RegisterPanel, TreeMap<Integer, BigIntegerBean>> regMap =
		new HashMap<RegisterPanel, TreeMap<Integer, BigIntegerBean>>();
	public HashMap<RegisterPanel, TreeMap<Integer, BigIntegerBean>> initRegMap =
		null; //stores original register/stone pairs after simulation.
	public TreeMap<Integer, BigIntegerBean> regs = new TreeMap<Integer, BigIntegerBean>(); // maps Integer -> BigInteger
	private final static BigInteger biMillion = new BigInteger("1000000");
	private final static BigIntegerBean BIBZero = new BigIntegerBean();
	private final static int regWidth = 60;
	private final static int INIT_HEIGHT = 70;
	private final static int INIT_WIDTH = 400;
	private JButton jump = new JButton("Jump To Register");
	private JButton newInput = new JButton("Add New Input Panel");
	private JButton removeInput = new JButton("Remove Input Panel");
	public static Color darkGray = new Color(32, 32, 32);
	public static Font normal = new Font("Verdana",Font.PLAIN,10);
	public static Font small = new Font("Verdana",Font.PLAIN,8);
	public static final int REG_ARC = 20;
	public static final int BUTTON_ARC = 15;
	public static final Color compBlue = new Color(0,146,255);
	public static final Color babyBlue = new Color(67,203,255);
	private static final int buttonTriangleHeightOffset = 10;
	
	private int selectedReg = -1;
	private boolean locked = false;
	private final String TITLE = "iPEAR - Improved Pleasing Editor for Abacus Registers";
	
	public RegistryEditorDialog()
	{
		setTitle(TITLE);
		
		this.inputPanel = new JPanel(new SpringLayout());
		addNewInputPanel();
		removeInput.setEnabled(false);
		//TODO: remove
		regPanel.addMouseListener(this);
		regPanel.addMouseMotionListener(this);
		
		setLayout(new BorderLayout());
		
		add(inputPanel, BorderLayout.CENTER);
		JPanel south = new JPanel();
		south.setBorder(BorderFactory.createLineBorder(Color.black));
		regPanel.setBackground(Color.blue);
		
		south.add(jump);
		south.add(newInput);
		south.add(removeInput);
		jump.addActionListener(this);
		newInput.addActionListener(this);
		removeInput.addActionListener(this);
		add(south,BorderLayout.SOUTH);
		pack();
		
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		
		setLocation(screenSize.width / 2 - getWidth() / 2,
				screenSize.height / 2 - getHeight() / 2);
		
		this.setDefaultCloseOperation(HIDE_ON_CLOSE);
	}
	//TODO: add action to create new register panel
	private void addNewInputPanel() {
		RegisterPanel pnl = new RegisterPanel(regMap.size() + 1);
		pnl.addMouseListener(this);
		pnl.addMouseMotionListener(this);
		pnl.setBackground(Color.BLUE);
		pnl.setBorder(BorderFactory.createLineBorder(Color.black));
		regMap.put(pnl, new TreeMap<Integer, BigIntegerBean>());
		inputPanel.add(pnl);
		SpringUtilities.makeCompactGrid(inputPanel, regMap.size(), 1, 0, 0, 0, 0);
		inputPanel.validate();
		inputPanel.repaint();
		
		if(regMap.size() > 1) {
			removeInput.setEnabled(true);
		}
	}
	
	private void removeInputPanel() {
		int code = 0; //will store the largest hash code
		RegisterPanel pnlToRemove = null;
		
		for(RegisterPanel pnl : regMap.keySet()) {
			if(code < pnl.hashCode()) {
				code = pnl.hashCode();
				pnlToRemove = pnl;
			}
		}
		
		if(pnlToRemove != null) {
			pnlToRemove.removeMouseListener(this);
			pnlToRemove.removeMouseMotionListener(this);
			regMap.remove(pnlToRemove);
			inputPanel.remove(pnlToRemove);
			SpringUtilities.makeCompactGrid(inputPanel, regMap.size(), 1, 0, 0, 0, 0);
			inputPanel.validate();
			inputPanel.repaint();
		}
		
		if(regMap.size() < 2) {
			removeInput.setEnabled(false);
		}
	}
	
	public void clearSelection()
	{
		selectedReg = -1;
		repaint();
	}
	
	/**
	 * lock the panel to prevent modification when Simulator panel is active.
	 */
	public void lock() {
		selectedReg = -1;
		locked = true;
		//To fully clone this collection, values need to be cloned as well.
		initRegMap = (HashMap<RegisterPanel, TreeMap<Integer, BigIntegerBean>>) regMap.clone();
		for(RegisterPanel pnl : regMap.keySet()) {
			initRegMap.put(pnl, (TreeMap<Integer, BigIntegerBean>) regMap.get(pnl).clone());
		}
		setTitle(TITLE + " (locked)");
	}
	
	public void unlock() {
		locked = false;
		regMap = (HashMap<RegisterPanel, TreeMap<Integer, BigIntegerBean>>) initRegMap.clone();
		for(RegisterPanel pnl : initRegMap.keySet()) {
			regMap.put(pnl, (TreeMap<Integer, BigIntegerBean>) initRegMap.get(pnl).clone());
		}
		repaint();
		setTitle(TITLE);
	}
	
	/**
	 * Add a pebble to this register
	 * @param reg the register number to add to
	 */
	public void addOne(int reg, RegisterPanel pnl)
	{
		BigIntegerBean contents = getRegisterContents(reg, pnl);
		setRegisterContents(reg,contents.getVal().add(BigInteger.ONE), pnl);
		selectedReg = reg;
		//repaint();
		this.inputPanel.getComponent(pnl.getPanelID() - 1).repaint();
	}
	
	/**
	 * Subtact one from this register
	 * @param reg the register to take one pebble out of
	 * @return false if we're empty
	 */
	public boolean subOne(int reg, RegisterPanel pnl)
	{
		boolean rv = false;
		BigIntegerBean contents = getRegisterContents(reg, pnl);
		
		if (!contents.getVal().equals(BigInteger.ZERO)) {
			rv = true;
			setRegisterContents(reg,contents.getVal().subtract(BigInteger.ONE), pnl);
		}
		
		selectedReg = reg;
		//repaint();
		this.inputPanel.getComponent(pnl.getPanelID() - 1).repaint();
		return rv;
	}
	
	public void setRegisterContents(int num, BigInteger i, RegisterPanel pnl) {
		TreeMap<Integer, BigIntegerBean> tmpMap = regMap.get(pnl);
		
		if(i.equals(BigInteger.ZERO))
			tmpMap.remove(new Integer(num));
		else
			tmpMap.put(num, new BigIntegerBean(i));
		regMap.put(pnl, tmpMap);
	}
	
	public BigIntegerBean getRegisterContents(int regNumber, RegisterPanel pnl) {
		BigIntegerBean val = this.regMap.get(pnl).get(regNumber);
		
		if(val == null) {
			val = BIBZero;
		}
		
		return val;
	}
	
	class RegisterPanel extends JPanel {		
		boolean leftOn = false, rightOn = false;
		private double curReg = 1.0;
		private final Integer panelID;
		
		public RegisterPanel(Integer id) {
			panelID = id;
			setPreferredSize(new Dimension(INIT_WIDTH * 2,INIT_HEIGHT * 2));
			
			RegTimer rt = new RegTimer();
			rt.start();
		}
		
		public Integer getPanelID() {
			return panelID;
		}
		
		protected void paintComponent(Graphics g) {
			super.paintComponent(g);
			Graphics2D g2d = (Graphics2D)g;
			setupDrawing(g);
			
			AffineTransform t = g2d.getTransform();
			
			//strechToFill(g2d);

			drawBins(g2d);
			
			g2d.setTransform(t);
			
			drawButtons(g2d);
		}

		private void drawBins(Graphics2D g) {
			long init = Math.round(Math.floor(curReg));
			double end = 1 + init + (this.getWidth()) / regWidth;
			double startX = (init - curReg) * regWidth + 40;
			g.setColor(Color.black);			
			
			for (long num = init; num < end;++num, startX += regWidth)
			{
				if (num > 0 && num <= Integer.MAX_VALUE)
				{
					int regNum = (int)num;
					BigInteger amount = getRegisterContents(regNum, this).getVal();
					
					RoundRectangle2D.Double r = new RoundRectangle2D.Double((int)(startX + 5), 5, regWidth - 10,this.getHeight() - 30,REG_ARC,REG_ARC);
					
					g.setColor(Color.white);
					g.fill(r);
					g.setColor((regNum == selectedReg && locked) ? Color.red : darkGray);
					g.draw(r);
					g.setFont(normal);
					
					if (regNum < 1000)
						g.drawString("Reg #" + regNum,(int)startX+10,this.getHeight()-9);
					else if (regNum < 1000000)
						g.drawString("R #" + regNum,(int)startX+10,this.getHeight()-9);
					else
					{						
						g.setFont(small);
						g.drawString("" + regNum,(int)startX+10,this.getHeight()-9);
						g.setFont(normal);
					}
					
					if (amount.min(biMillion).equals(biMillion))
					{
						g.drawString(">999999",(int)startX+8,(int)r.getCenterY() + 5);	
					}
					else
						g.drawString(amount.toString(),(int)startX+10,(int)r.getCenterY() + 5);					
				}
			}
		}
		
		private void strechToFill(Graphics2D g)
		{
			double scaleX = (double)getWidth() / INIT_WIDTH;
			double scaleY = (double)getHeight() / INIT_HEIGHT;
			
			g.scale(scaleX,scaleY);
		}
		
		private void drawButtonsUnder(boolean first,Graphics2D g)
		
		{
			int h = this.getHeight() - 1;
			
			if (first && leftOn)
			{
				Polygon p = new Polygon();
				p.addPoint(scrollButtonWidth + 1,buttonTriangleHeightOffset);
				p.addPoint(buttonTriangleHeightOffset + 1,h/2);
				p.addPoint(scrollButtonWidth + 1,h-buttonTriangleHeightOffset);
				
				g.setColor(babyBlue);
				g.fill(p);
			}
			
			if (!first && rightOn)
			{
				Polygon p = new Polygon();
				p.addPoint(0,buttonTriangleHeightOffset);
				p.addPoint(scrollButtonWidth-buttonTriangleHeightOffset,h/2);
				p.addPoint(0,h-buttonTriangleHeightOffset);
				
				g.setColor(babyBlue);
				g.fill(p);				
			}
		}
		
		private Image oneImage = null;
		private boolean lastOnOne = false;
		
		private Image getImageOne()
		{
			int h = this.getHeight() - 1;
			int imW = scrollButtonWidth+1;
			int imH = h+1;
			Image rv;
			
			if (oneImage != null && oneImage.getWidth(null) == imW 
					&& oneImage.getHeight(null) == imH && lastOnOne == leftOn)
			{
				rv = oneImage;
			}
			else
			{		
				Image button = createImage(imW,imH);
				Graphics2D g = (Graphics2D)button.getGraphics();
				setupDrawing(g);
				
				g.setColor(babyBlue);
				g.fillRect(0,0,button.getWidth(null),button.getHeight(null));
				
				RoundRectangle2D.Double one = new RoundRectangle2D.Double(0,0,scrollButtonWidth,h,BUTTON_ARC,BUTTON_ARC);
	
				g.setColor(Color.lightGray);
				g.fill(one);
				
				g.setColor(Color.black);
				g.draw(one);
				
				Polygon p = new Polygon();
				p.addPoint(scrollButtonWidth,buttonTriangleHeightOffset);
				p.addPoint(buttonTriangleHeightOffset,h/2);
				p.addPoint(scrollButtonWidth,h-buttonTriangleHeightOffset);
				
				g.setColor(Color.black);
				g.drawPolyline(p.xpoints,p.ypoints,p.npoints);
				
				drawButtonsUnder(true,g);
				
				button = ButtonPanel.makeTransparent(button,babyBlue);
				oneImage = rv = button;
				lastOnOne = leftOn;
			}
			
			return rv;
		}
		
		private Image twoImage = null;
		private boolean lastOnTwo = false;
		
		private Image getImageTwo()
		{
			int h = this.getHeight() - 1;
			int imW = scrollButtonWidth+1;
			int imH = h+1;
			Image rv;
			
			if (twoImage != null && twoImage.getWidth(null) == imW 
					&& twoImage.getHeight(null) == imH && lastOnTwo == rightOn)
			{
				rv = twoImage;
			}
			else
			{		
				Image button = createImage(scrollButtonWidth+1,h+1);
				Graphics2D g = (Graphics2D)button.getGraphics();
				setupDrawing(g);
				
				g.setColor(babyBlue);
				g.fillRect(0,0,button.getWidth(null),button.getHeight(null));
				
				RoundRectangle2D.Double two = new RoundRectangle2D.Double(0,0,scrollButtonWidth,h,BUTTON_ARC,BUTTON_ARC);
				
				g.setColor(Color.lightGray);
				g.fill(two);
				
				g.setColor(Color.black);
				g.draw(two);
				
				Polygon p = new Polygon();			
				p.addPoint(0,buttonTriangleHeightOffset);
				p.addPoint(scrollButtonWidth-buttonTriangleHeightOffset,h/2);
				p.addPoint(0 ,h-buttonTriangleHeightOffset);
				
				g.setColor(Color.black);
				g.drawPolyline(p.xpoints,p.ypoints,p.npoints);
				
				drawButtonsUnder(false,g);
				rv = twoImage = button = ButtonPanel.makeTransparent(button,babyBlue);
				lastOnTwo = rightOn;
			}
			
			return rv;
		}

		private void drawButtons(Graphics2D graphics)
		{			
			int w = this.getWidth();
			
			graphics.drawImage(getImageOne(),0,0,null);
			
			// two
			graphics.drawImage(getImageTwo(),w-scrollButtonWidth,0,null);
		}
		
		public void resetButtons()
		{
			leftOn = rightOn = false;
		}
		
		private void setupDrawing(Graphics g)
		{
			Graphics2D g2 = (Graphics2D)g;
			// Enable Anti-Aliasing
			g2.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON);   
			g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		}
		
		class RegTimer extends Thread
		{
			public void run() 
			{				
				long lastTime = -1;
				long held = 0;
				
				while (true)
				{					
					if (leftOn || rightOn)
					{
						long time  = new Date().getTime();
						
						if (lastTime != -1)
						{
							double div = Math.max(500-(held/7),100);
							long dif = time - lastTime;
							double dx = dif / div;
							
							if (rightOn)
							{
								curReg += dx;
								
								if (curReg > Integer.MAX_VALUE)
									curReg = Integer.MAX_VALUE;
							}
							else if (leftOn)
							{
								curReg -= dx;
								
								if (curReg < 1)
									curReg = 1;
							}
							
							held += dif;
							repaint();
						}
						
						lastTime = time;
					}
					else
					{
						lastTime = -1;
						held = 0;
					}
						
					try
					{						
						Thread.sleep(10);
					}
					catch (Exception e) { }
				}
			}
		}
	}
	
	private RegisterPanel getSelectedComponent(Object source) {
		if(source instanceof RegisterPanel) {
			for(RegisterPanel pnl : regMap.keySet()) {
				if(((RegisterPanel)source) == pnl)
					return pnl;
			}
		}
		return null;
	}

	public void mouseDragged(MouseEvent arg0) { }

	public void mouseMoved(MouseEvent e) {
		RegisterPanel pnl = getSelectedComponent(e.getSource());
		Point p = e.getPoint();
		int w = pnl.getWidth();
		int h = pnl.getHeight();
		
		RoundRectangle2D.Double left = new RoundRectangle2D.Double(0,0,scrollButtonWidth,h,BUTTON_ARC,BUTTON_ARC);
		RoundRectangle2D.Double right = new RoundRectangle2D.Double(w-scrollButtonWidth,0,scrollButtonWidth,h,BUTTON_ARC,BUTTON_ARC);
		
		if (left.contains(p))
		{
			if (pnl.leftOn == false || pnl.rightOn == true)
			{
				pnl.leftOn = true;
				pnl.rightOn = false;
				pnl.repaint();
			}			
		}
		else if (right.contains(p))
		{
			if (pnl.leftOn == true || pnl.rightOn == false)
			{
				pnl.leftOn = false;
				pnl.rightOn = true;
				pnl.repaint();
			}
		}
		else
		{
			if (pnl.leftOn == true || pnl.rightOn == true)
			{
				pnl.resetButtons();
				pnl.repaint();
			}
		}		
	}
	
	// -1 if none
	public int getClickedRegister(Point p, RegisterPanel pnl) {
		long init = Math.round(Math.floor(pnl.curReg));
		double end = 1 + init + (pnl.getWidth()) / regWidth;
		double startX = (init - pnl.curReg) * regWidth + 40;
		int rv = -1;
		
		for (long num = init; num < end && num >= 0;++num, startX += regWidth)
		{
			if (num <= Integer.MAX_VALUE)
			{
				int regNum = (int)num;					
				
				RoundRectangle2D.Double r = new RoundRectangle2D.Double((int)(startX + 5), 5, regWidth - 10, pnl.getHeight() - 30 ,REG_ARC,REG_ARC);

				if (r.contains(p))
				{
					rv = regNum;
					break;
				}
			}
		}
		
		return rv;
	}
	
	// trnaslate x
	public double scaleX(double from)
	{
		double scaleX = INIT_WIDTH / (double)regPanel.getWidth();
		
		return from * scaleX;
	}
	
	// translate real coords to stetched coords
	public Point translatePoint(Point from)
	{
		double scaleX = INIT_WIDTH / (double)regPanel.getWidth();
		double scaleY = INIT_HEIGHT / (double)regPanel.getHeight();
		
		return new Point((int)(from.x * scaleX), (int)(from.y * scaleY));
	}

	public void mouseClicked(MouseEvent e){}

	public void mousePressed(MouseEvent e) {
		if (!locked && e.getButton() == MouseEvent.BUTTON1) {
			Point p = e.getPoint();
			RegisterPanel pnl = getSelectedComponent(e.getSource());
			
			if (p.x > scrollButtonWidth && p.x < pnl.getWidth() - scrollButtonWidth) {
				
				int clicked = getClickedRegister(p, pnl);
				
				if (clicked != -1)
				{
					BigInteger cur = getRegisterContents(clicked, pnl).getVal();
					
					String s = (String)JOptionPane.showInputDialog(
		                    null,
		                    "What would you like to set this register to?",
		                    "Register " + clicked,
		                    JOptionPane.PLAIN_MESSAGE,
		                    null,
		                    null,
		                    cur.toString());

					//If a string was returned
					if ((s != null) && (s.length() > 0)) {
					    try
					    {
					    	BigInteger bi = new BigInteger(s);
					    	setRegisterContents(clicked, bi, pnl);
					    	repaint();
					    }
					    catch (NumberFormatException er)
					    {
					    	JOptionPane.showMessageDialog(null, "You didn't enter an integer: '" + s + "'");
					    }
					    
					    return;
					}
					
					
				}
			}
		}
	}

	public void mouseReleased(MouseEvent arg0) { }

	public void mouseEntered(MouseEvent arg0) { }

	public void mouseExited(MouseEvent e) {
		RegisterPanel pnl = getSelectedComponent(e.getSource());
		if (pnl.leftOn == true || pnl.rightOn == true) {
			pnl.resetButtons();
			pnl.repaint();
		}
	}

	public void actionPerformed(ActionEvent e) {
		if(!locked) { //only modify the dialog when it is not locked.
			if (e.getSource() == jump) {
				String s = (String)JOptionPane.showInputDialog(
	                    null,
	                    "Which register would you like to jump to?",
	                    "Enter Register Number",
	                    JOptionPane.PLAIN_MESSAGE,
	                    null,
	                    null,
	                    "" + Math.round(0));
	
				//If a string was returned
				if ((s != null) && (s.length() > 0)) {
				    try {
				    	int i = Integer.parseInt(s);
				    	for(RegisterPanel pnl : regMap.keySet()) {
				    		pnl.curReg = Math.max(0,i);
				    	}
				    	repaint();
				    }
				    catch (NumberFormatException er)
				    {
				    	JOptionPane.showMessageDialog(null, "You didn't enter an integer between 0 and " +
				    			Integer.MAX_VALUE + ": '" + s + "'");
				    }
				    
				    return;
				}
			} else if (e.getSource() == newInput) {
				this.addNewInputPanel();
			} else if (e.getSource() == removeInput) {
				this.removeInputPanel();
			}
		} else {
			JOptionPane.showMessageDialog(this, "ERROR: Please unlock panel (change currently selected tab) before changing the registers",
					"Error", JOptionPane.ERROR_MESSAGE);
		}
	}
	public int getPanelCount() {
		int count = (regMap.size() < 1) ? 1 : regMap.size();
		return count;
	}
	
	public Set<RegisterPanel> getPanels() {
		return regMap.keySet();
	}
}
