package com.abacus.improved;

import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.Set;
import java.util.TreeMap;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import com.abacus.improved.RegistryEditorDialog.RegisterPanel;
import com.abacus.legacy.Node;
/**
 * The purpose of this class is to interface with the Registry and the Node editor panels in order
 * to display changes to the machine based on the given input. The number of input panels in the register dialog
 * determines the number of threads running in the background.
 * @author saiumesh
 *
 */
public class SimulatorPanel extends JPanel implements ActionListener, MouseListener {
	protected JLabel numSteps = new JLabel("Number of Steps: 0");
	protected JLabel numRegisters = new JLabel("Number of Registers: 0");
	protected JButton startSim, endSim, stepSim;
	protected ArrayList<Node> nodeList;
	private ArrayList<Player> simList; //list of threads that execute different inputs for the given machine
	private TreeMap<String, Double> simSpeedMap;
	private JComboBox speedBox, inputViewBox;
	private final String[] speedArray = {"Step", "Slow", "Normal", "Fast", "Ludicrous"};
	private Double speedMultiplier;
	protected boolean waitForStep = true, stopThread = false;
	RegistryEditorDialog regEdit;
	private MachinePanel nodeEdit;
	
	public SimulatorPanel(RegistryEditorDialog re) {
		this.regEdit = re;
		this.nodeEdit = null;
		this.startSim = new JButton("Start");
		this.endSim = new JButton("Stop");
		this.stepSim = new JButton("Step");
		simSpeedMap = new TreeMap<String, Double>();
		speedBox = new JComboBox(speedArray);
		speedBox.setSelectedIndex(2);
		
		speedBox.addActionListener(this);
		startSim.addActionListener(this);
		endSim.addActionListener(this);
		stepSim.addActionListener(this);
		inputViewBox = new JComboBox();
		simSpeedMap.put(speedArray[0], null);
		simSpeedMap.put(speedArray[1], 2.0); //Thread.sleep multipliers (base time is 1000 ms)
		simSpeedMap.put(speedArray[2], 1.0);
		simSpeedMap.put(speedArray[3], 0.5);
		simSpeedMap.put(speedArray[4], 0.1);
		
		speedMultiplier = 1.0;
		addComponentsToPanel();
		initThreadList();
	}
	public void reinitPanel() {
		initThreadList();
	}
	private void initThreadList() {
		if(simList != null && !this.isRunning()) {
			simList.removeAll(simList);//reinit list
			inputViewBox.removeAll();
			Set<RegisterPanel> pnlList = regEdit.getPanels();
			int idx = 1;
			for(RegisterPanel pnl : pnlList) {
				simList.add(new Player(idx, pnl, nodeList));
				inputViewBox.addItem("Panel " + idx);
				idx++;
			}
		}
	}
	
	private void addComponentsToPanel() {
		this.setLayout(new GridLayout(2, 3));
		add(startSim);
		add(endSim);
		add(inputViewBox);
		add(numSteps);
		add(numRegisters);
		add(speedBox);
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if(e.getSource() == speedBox) {
			speedMultiplier = simSpeedMap.get((String) speedBox.getSelectedItem());
			if (speedMultiplier == null) {
				startSim.setText("Step");
				waitForStep = true;
			} else {
				startSim.setText("Start");
				waitForStep = false;
			}
		} else if (e.getSource() == endSim) {
			if (endSim.getText() == "Stop") {
				stopThread = true;
				endSim.setText("Reset");
			} else if (endSim.getText() == "Reset") {
				regEdit.unlock();
				regEdit.lock();
				nodeEdit.unlockPanel();
				regEdit.clearSelection();
				endSim.setText("Stop");
			}
		} else if(e.getSource() == startSim) {
			if (endSim.getText() == "Stop") {
				nodeEdit.lockPanel();
				stopThread = false;
				this.stepSim.setEnabled(false);
				endSim.setEnabled(true);
				if(startSim.getText().equals("Start")) {
					for(int i = 0; i < simList.size(); i++) {
						if(!simList.get(i).isAlive())
							simList.get(i).start();
					}
				} else if (startSim.getText().equals("Step")) {
					for(int i = 0; i < simList.size(); i++) {
						if(!simList.get(i).isAlive() && !simList.get(i).isDone())
							simList.get(i).start();
					}
					waitForStep = false;
				} else {
					JOptionPane.showMessageDialog(new JFrame(), "Please reset the simulation before continuing",
							"Error", JOptionPane.ERROR_MESSAGE);
				}
			}
		}
	}

	public void setNodeList(ArrayList<Node> nodeList) {
		this.nodeList = nodeList;
	}
	
	/**
	 * check to see if any of the Player threads are still running.
	 * @return
	 */
	public boolean isRunning() {
		if (simList != null) {
			Thread t = null;
			for (int i = 0; i < simList.size(); i++) {
				t = simList.get(i);
				if(t != null && t.isAlive())
					return true;
			}
		}
		
		return false;
	}
	/**
	 * This inner class takes the registry and node map information and runs the machine
	 * @author saiumesh
	 *
	 */
	private class Player extends Thread {
		private Integer assignedPanelId;
		private RegisterPanel assignedPanel;
		private Node curNode = null;
		private Node previousNode = null;
		private int stepCount = 0;
		private ArrayList<Node> nodes;
		private boolean isDone = false;
		public Player(int pnlIndex, RegisterPanel pnl, ArrayList<Node> nodes) {
			this.assignedPanelId = pnlIndex;
			this.assignedPanel = pnl;
			this.nodes = nodes;
			this.curNode = nodes.get(0);
		}
		
		private void step() {
			stepCount++;
			
			curNode.simSelect(Node.SELECTED_NODE);
			if (assignedPanelId == (inputViewBox.getSelectedIndex() + 1)) {
				//System.out.println("PING 1");
				nodeEdit.editor.updateNodeList(curNode);
				nodeEdit.repaint();
				numSteps.setText("Number of Steps: " + stepCount);
			}
			try {
				Thread.sleep((long) (speedMultiplier * 1000));
			} catch (Exception e) { }
			curNode.simSelect(Node.SELECTED_NONE);
			// do the transition	
			Node nextNode = null;
			
			int reg = curNode.getRegister();
			
			if (curNode.isPlus()) {
				regEdit.addOne(reg, this.assignedPanel);
				curNode.simSelect(Node.SELECTED_OUT);
				nextNode = curNode.getOut();
			} else {
				if (!regEdit.subOne(reg, assignedPanel)) { // empty
					curNode.simSelect(Node.SELECTED_OUTEMPTY);
					nextNode = curNode.getOutEmpty();
				}
				else {
					curNode.simSelect(Node.SELECTED_OUT);
					nextNode = curNode.getOut();
				}
			}
			
			if (assignedPanelId == (inputViewBox.getSelectedIndex() + 1)) {
				nodeEdit.editor.updateNodeList(curNode);
				nodeEdit.repaint();
				numSteps.setText("Number of Steps: " + stepCount);
			}
			
			try {
				Thread.sleep((long) (speedMultiplier * 50));
			} catch (Exception e) { }
			
			if (nextNode != null) {
				curNode.simSelect(Node.SELECTED_NONE);
			}
			previousNode = curNode;
			curNode = nextNode;
			
			if (curNode != null && curNode.isPauseState()) { // pause!
				curNode.simSelect(Node.SELECTED_NODE);
			}
		}

		public void run() {
			while(!stopThread) {
				//do stuff
				if (curNode == null) {
					break;
				} else {
					step();
					//conditions for simulation speed
					if (speedMultiplier == null) {
						synchronized(this) {
							try {
								while(waitForStep) {
									this.sleep(1);
								}
								waitForStep = true;
							} catch(InterruptedException e) {
								e.printStackTrace();
							}
						}
					} else {
						try {
							Thread.sleep((long) (speedMultiplier * 1000));
						} catch (InterruptedException e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}
					}
				}
			}
			isDone = true;
		}
		//prevent thread exception from being thrown following termination
		public boolean isDone() {
			return isDone;
		}
	}

	@Override
	public void mouseClicked(MouseEvent arg0) { }

	@Override
	public void mouseEntered(MouseEvent arg0) { }

	@Override
	public void mouseExited(MouseEvent arg0) { }

	@Override
	public void mousePressed(MouseEvent e) {}

	@Override
	public void mouseReleased(MouseEvent arg0) { }
	
	/**
	 * reinit the thread list for new inputs
	 * @param dialog
	 */
	public void setDialog(RegistryEditorDialog dialog) {
		this.regEdit = dialog;
		this.simList = new ArrayList<Player>();
		this.initThreadList();
	}
	
	public void setParent(MachinePanel nodeEdit) {
		this.nodeEdit = nodeEdit;
	}

}
